@@ -11,6 +11,7 @@ import time
 import zlib
 
 import _damo_fmt_str
+import _damo_subproc
 import _damon
 import _damon_args
 import damo_report_access
@@ -375,8 +376,7 @@ def set_perf_path(perf_path):
 
     # Test perf record for damon event
     err = None
-    try:
-        subprocess.check_output(['which', PERF])
+    if _damo_subproc.avail_cmd(PERF):
         try:
             subprocess.check_output(
                     [PERF, 'record', '-e', perf_event_damon_aggregated, '--',
@@ -384,7 +384,7 @@ def set_perf_path(perf_path):
                     stderr=subprocess.PIPE)
         except:
             err = 'perf record not working with "%s"' % PERF
-    except:
+    else:
         err = 'perf not found at "%s"' % PERF
     return err
 
@@ -408,6 +408,9 @@ def parse_records_file(record_file, monitoring_intervals=None):
     Return monitoring results records and error string
     '''
 
+    if not _damo_subproc.avail_cmd('file'):
+        return None, "'file' command seems not installed"
+
     file_type = subprocess.check_output(
             ['file', '-b', record_file]).decode().strip()
     if file_type == 'JSON data':
@@ -1301,6 +1304,13 @@ def update_get_snapshot_records(kdamond_idxs, scheme_idxs,
     while err is not None and nr_tries < 5:
         nr_tries += 1
 
+        # todo: update tuned intervals only if auto-tuning is ongoing
+        err = _damon.update_tuned_intervals(kdamond_idxs)
+        if err is not None:
+            time.sleep(random.randrange(
+                2**(nr_tries - 1), 2**nr_tries) / 100)
+            continue
+
         err = _damon.update_schemes_tried_regions(kdamond_idxs)
 
         if err is not None:
