@@ -17,21 +17,24 @@ import damo_version
 
 # Core data structures
 
-class DamonIntervals:
-    sample = None
-    aggr = None
-    ops_update = None
+class DamonIntervalsGoal:
+    access_bp = None
+    aggrs = None
+    min_sample_us = None
+    max_sample_us = None
 
-    def __init__(self, sample='5ms', aggr='100ms', ops_update='1s'):
-        self.sample = _damo_fmt_str.text_to_us(sample)
-        self.aggr = _damo_fmt_str.text_to_us(aggr)
-        self.ops_update = _damo_fmt_str.text_to_us(ops_update)
+    def __init__(self, access_bp='0%', aggrs=0, min_sample_us=0, max_sample_us=0):
+        self.access_bp = _damo_fmt_str.text_to_bp(access_bp)
+        self.aggrs = _damo_fmt_str.text_to_nr(aggrs)
+        self.min_sample_us = _damo_fmt_str.text_to_us(min_sample_us)
+        self.max_sample_us = _damo_fmt_str.text_to_us(max_sample_us)
 
     def to_str(self, raw):
-        return 'sample %s, aggr %s, update %s' % (
-                _damo_fmt_str.format_time_us(self.sample, raw),
-                _damo_fmt_str.format_time_us(self.aggr, raw),
-                _damo_fmt_str.format_time_us(self.ops_update, raw))
+        return 'target %s accesses per %s aggrs, [%s, %s] sampling interval' % (
+                _damo_fmt_str.format_bp(self.access_bp, raw),
+                _damo_fmt_str.format_nr(self.aggrs, raw),
+                _damo_fmt_str.format_time_us(self.min_sample_us, raw),
+                _damo_fmt_str.format_time_us(self.max_sample_us, raw))
 
     def __str__(self):
         return self.to_str(False)
@@ -41,17 +44,71 @@ class DamonIntervals:
 
     @classmethod
     def from_kvpairs(cls, kvpairs):
-        return DamonIntervals(
-                kvpairs['sample_us'], kvpairs['aggr_us'],
-                kvpairs['ops_update_us'])
+        return DamonIntervalsGoal(
+                kvpairs['access_bp'], kvpairs['aggrs'],
+                kvpairs['min_sample_us'], kvpairs['max_sample_us'])
 
     def to_kvpairs(self, raw=False):
         return collections.OrderedDict([
+            ('access_bp', _damo_fmt_str.format_bp(self.access_bp, raw)),
+            ('aggrs', _damo_fmt_str.format_nr(self.aggrs, raw)),
+            ('min_sample_us', _damo_fmt_str.format_time_us(
+                self.min_sample_us, raw)),
+            ('max_sample_us', _damo_fmt_str.format_time_us(
+                self.max_sample_us, raw)),
+            ])
+
+    def enabled(self):
+        return self.aggrs != 0
+
+class DamonIntervals:
+    sample = None
+    aggr = None
+    ops_update = None
+    intervals_goal = None
+
+    def __init__(self, sample='5ms', aggr='100ms', ops_update='1s',
+                 intervals_goal=DamonIntervalsGoal()):
+        self.sample = _damo_fmt_str.text_to_us(sample)
+        self.aggr = _damo_fmt_str.text_to_us(aggr)
+        self.ops_update = _damo_fmt_str.text_to_us(ops_update)
+        self.intervals_goal = intervals_goal
+
+    def to_str(self, raw):
+        lines = ['sample %s, aggr %s, update %s' % (
+                _damo_fmt_str.format_time_us(self.sample, raw),
+                _damo_fmt_str.format_time_us(self.aggr, raw),
+                _damo_fmt_str.format_time_us(self.ops_update, raw))]
+        if self.intervals_goal.enabled():
+            lines.append('%s' % self.intervals_goal.to_str(raw))
+        return '\n'.join(lines)
+
+    def __str__(self):
+        return self.to_str(False)
+
+    def __eq__(self, other):
+        return type(self) == type(other) and '%s' % self == '%s' % other
+
+    @classmethod
+    def from_kvpairs(cls, kvpairs):
+        if not 'intervals_goal' in kvpairs:
+            return DamonIntervals(
+                    kvpairs['sample_us'], kvpairs['aggr_us'],
+                    kvpairs['ops_update_us'])
+        return DamonIntervals(
+                kvpairs['sample_us'], kvpairs['aggr_us'],
+                kvpairs['ops_update_us'],
+                DamonIntervalsGoal.from_kvpairs(kvpairs['intervals_goal']))
+
+    def to_kvpairs(self, raw=False, omit_defaults=False):
+        kvp = collections.OrderedDict([
             ('sample_us', _damo_fmt_str.format_time_us(self.sample, raw)),
             ('aggr_us', _damo_fmt_str.format_time_us(self.aggr, raw)),
             ('ops_update_us',
-                _damo_fmt_str.format_time_us(self.ops_update, raw)),
-            ])
+                _damo_fmt_str.format_time_us(self.ops_update, raw))])
+        if not omit_defaults or self.intervals_goal != DamonIntervalsGoal():
+            kvp['intervals_goal'] = self.intervals_goal.to_kvpairs(raw)
+        return kvp
 
 class DamonNrRegionsRange:
     minimum = None
@@ -629,21 +686,25 @@ class DamosWatermarks:
                 ])
 
 class DamosFilter:
-    filter_type = None  # anon, memcg, young, hugepage, addr, or target
+    filter_type = None  # anon, memcg, young, hugepage_size, unmapped, addr, or
+                        # target
     matching = None
     allow = None
     memcg_path = None
     address_range = None    # DamonRegion
+    hugepage_size = None    # hugepage size in bytes
     damon_target_idx = None
     scheme = None
 
     def __init__(self, filter_type, matching, allow=False,
-                 memcg_path=None, address_range=None, damon_target_idx=None):
+                 memcg_path=None, address_range=None, damon_target_idx=None,
+                 hugepage_size=None):
         self.filter_type = filter_type
         self.matching = _damo_fmt_str.text_to_bool(matching)
         self.memcg_path = memcg_path
         self.allow = _damo_fmt_str.text_to_bool(allow)
         self.address_range = address_range
+        self.hugepage_size = hugepage_size
         if damon_target_idx != None:
             self.damon_target_idx = _damo_fmt_str.text_to_nr(damon_target_idx)
 
@@ -656,7 +717,7 @@ class DamosFilter:
         if self.matching is False:
             words.append('none')
         words.append(self.filter_type)
-        if self.filter_type in ['anon', 'young', 'hugepage']:
+        if self.filter_type in ['anon', 'young', 'unmapped']:
             return ' '.join(words)
         if self.filter_type == 'memcg':
             return ' '.join(words + [self.memcg_path])
@@ -665,6 +726,8 @@ class DamosFilter:
         if self.filter_type == 'target':
             return ' '.join(words + [_damo_fmt_str.format_nr(
                     self.damon_target_idx, raw)])
+        if self.filter_type == 'hugepage_size':
+            return ' '.join(words + [self.hugepage_size.to_str(raw)])
 
     def __str__(self):
         return self.to_str(False)
@@ -703,7 +766,8 @@ class DamosFilter:
 
     def handled_by_ops(self):
         # whether this filter is handled by DAMON operations set layer
-        return self.filter_type in ['anon', 'memcg', 'young', 'hugepage']
+        return self.filter_type in ['anon', 'memcg', 'young', 'hugepage_size',
+                                    'unmapped']
 
 class DamosStats:
     nr_tried = None
@@ -897,20 +961,24 @@ class Damos:
                 filters,
                 None, None)
 
-    def to_kvpairs(self, raw=False):
+    def to_kvpairs(self, raw=False, omit_defaults=False, params_only=False):
         kv = collections.OrderedDict()
         kv['action'] = self.action
         if is_damos_migrate_action(self.action):
             kv['target_nid'] = self.target_nid
-        kv['access_pattern'] = self.access_pattern.to_kvpairs(raw)
+        if not omit_defaults or self.access_pattern != DamosAccessPattern():
+            kv['access_pattern'] = self.access_pattern.to_kvpairs(raw)
         kv['apply_interval_us'] = self.apply_interval_us
-        kv['quotas'] = self.quotas.to_kvpairs(raw)
-        kv['watermarks'] = self.watermarks.to_kvpairs(raw)
-        filters = []
-        for damos_filter in self.filters:
-            filters.append(damos_filter.to_kvpairs(raw))
-        kv['filters'] = filters
-        if self.stats != None:
+        if not omit_defaults or self.quotas != DamosQuotas():
+            kv['quotas'] = self.quotas.to_kvpairs(raw)
+        if not omit_defaults or self.watermarks != DamosWatermarks():
+            kv['watermarks'] = self.watermarks.to_kvpairs(raw)
+        if not omit_defaults or self.filters != []:
+            filters = []
+            for damos_filter in self.filters:
+                filters.append(damos_filter.to_kvpairs(raw))
+            kv['filters'] = filters
+        if not params_only and self.stats is not None:
             kv['stats'] = self.stats.to_kvpairs(raw)
         return kv
 
@@ -951,7 +1019,13 @@ class DamonCtx:
         for idx, target in enumerate(self.targets):
             lines.append('target %d' % idx)
             lines.append(_damo_fmt_str.indent_lines(target.to_str(raw), 4))
-        lines.append('intervals: %s' % self.intervals.to_str(raw))
+        intervals_lines = self.intervals.to_str(raw).split('\n')
+        if len(intervals_lines) == 1:
+            lines.append('intervals: %s' % intervals_lines[0])
+        else:
+            lines.append('intervals')
+            lines.append(
+                    _damo_fmt_str.indent_lines(self.intervals.to_str(raw), 4))
         lines.append('nr_regions: %s' % self.nr_regions.to_str(raw))
         for idx, scheme in enumerate(self.schemes):
             lines.append('scheme %d' % idx)
@@ -980,13 +1054,16 @@ class DamonCtx:
                     if 'schemes' in kv else [])
         return ctx
 
-    def to_kvpairs(self, raw=False):
+    def to_kvpairs(self, raw=False, omit_defaults=False, params_only=False):
         kv = collections.OrderedDict({})
         kv['ops'] = self.ops
         kv['targets'] = [t.to_kvpairs(raw) for t in self.targets]
-        kv['intervals'] = self.intervals.to_kvpairs(raw)
-        kv['nr_regions'] = self.nr_regions.to_kvpairs(raw)
-        kv['schemes'] = [s.to_kvpairs(raw) for s in self.schemes]
+        if not omit_defaults or self.intervals != DamonIntervals():
+            kv['intervals'] = self.intervals.to_kvpairs(raw)
+        if not omit_defaults or self.nr_regions != DamonNrRegionsRange():
+            kv['nr_regions'] = self.nr_regions.to_kvpairs(raw)
+        kv['schemes'] = [s.to_kvpairs(raw, omit_defaults, params_only)
+                         for s in self.schemes]
         return kv
 
 def target_has_pid(ops):
@@ -1049,11 +1126,13 @@ class Kdamond:
                 kv['pid'] if 'pid' in kv else None,
                 [DamonCtx.from_kvpairs(c) for c in kv['contexts']])
 
-    def to_kvpairs(self, raw=False):
+    def to_kvpairs(self, raw=False, omit_defaults=False, params_only=False):
         kv = collections.OrderedDict()
-        kv['state'] = self.state
-        kv['pid'] = self.pid
-        kv['contexts'] = [c.to_kvpairs(raw) for c in self.contexts]
+        if not params_only:
+            kv['state'] = self.state
+            kv['pid'] = self.pid
+        kv['contexts'] = [c.to_kvpairs(raw, omit_defaults, params_only)
+                          for c in self.contexts]
         return kv
 
 import _damo_fs
@@ -1113,10 +1192,14 @@ features = ['record',       # was in DAMON patchset, but not merged in mainline
             'schemes_quota_goal_some_psi',  # merged in v6.9-rc1
             'schemes_filters_young',    # merged in v6.10-rc1
             'schemes_migrate',          # merged in v6.11-rc1
-            'sz_ops_filter_passed',     # developing
-            'pass_filter',              # was developing, renamed to
-                                        # pass_filter after v1 patch
-            'allow_filter',             # developing
+            'sz_ops_filter_passed',     # merged in v6.14-rc1
+            'allow_filter',             # merged in v6.14-rc1
+            'schemes_filters_hugepage_size',
+                                        # merged in mm-unstable
+            'schemes_filters_unmapped', # merged in mm-unstable
+            'intervals_goal',           # under development
+            'schemes_filters_core_ops_dirs',
+                                        # under development
             ]
 
 _damon_fs = None
@@ -1325,6 +1408,17 @@ def commit(kdamonds, commit_quota_goals_only=False):
         return 'commit staged updates filed (%s)' % err
     return None
 
+def update_tuned_intervals(kdamond_idxs=None):
+    if kdamond_idxs is None:
+        kdamond_idxs = running_kdamond_idxs()
+    if _damon_fs == _damon_dbgfs:
+        return None
+    err = _damon_fs.update_tuned_intervals(kdamond_idxs)
+    # 'Invalid argument' means the feature is not supported on the kernel
+    if err is not None and not 'Invalid argument' in err:
+        return err
+    return None
+
 def update_schemes_stats(kdamond_idxs=None):
     if kdamond_idxs == None:
         kdamond_idxs = running_kdamond_idxs()
@@ -1396,11 +1490,19 @@ def is_kdamond_running(kdamond_idx):
 def current_kdamonds():
     return _damon_fs.current_kdamonds()
 
-def update_read_kdamonds(nr_retries=0, update_stats=True,
-        update_tried_regions=True, update_quota_effective_bytes=False):
+def update_read_kdamonds(
+        nr_retries=0, update_stats=True, update_tried_regions=True,
+        update_quota_effective_bytes=False, do_update_tuned_intervals=False):
     err = 'assumed error'
     nr_tries = 0
     while True:
+        if do_update_tuned_intervals:
+            err = update_tuned_intervals()
+            if err is not None:
+                nr_tries += 1
+                time.sleep(
+                        random.randrange(2**(nr_tries - 1), 2**nr_tries) / 100)
+                continue
         err = update_schemes_status(update_stats, update_tried_regions,
                                     update_quota_effective_bytes)
         nr_tries += 1
