@@ -17,8 +17,9 @@ except ModuleNotFoundError as e:
     # properly.
     pass
 
-import _damo_paddr_layout
+import _damo_subproc
 import _damon
+import damo_pa_layout
 
 # Kdamonds construction from command line arguments
 
@@ -50,13 +51,13 @@ def init_regions_for(args):
 
     if args.ops == 'paddr' and not init_regions:
         if args.numa_node != None:
-            init_regions, err = _damo_paddr_layout.paddr_region_of(
+            init_regions, err = damo_pa_layout.numa_addr_ranges(
                     args.numa_node)
             if err != None:
                 return None, err
             init_regions = merge_cont_ranges(init_regions)
         else:
-            init_regions = [_damo_paddr_layout.default_paddr_region()]
+            init_regions = [damo_pa_layout.default_paddr_region()]
         try:
             init_regions = [_damon.DamonRegion(r[0], r[1])
                     for r in init_regions]
@@ -67,8 +68,11 @@ def init_regions_for(args):
 
 def damon_intervals_for(args):
     default_intervals = _damon.DamonIntervals()
-    intervals1 = _damon.DamonIntervals(args.sample, args.aggr, args.updr)
-    intervals2 = _damon.DamonIntervals(*args.monitoring_intervals)
+    intervals_goal = _damon.DamonIntervalsGoal(*args.monitoring_intervals_goal)
+    intervals1 = _damon.DamonIntervals(args.sample, args.aggr, args.updr,
+                                       intervals_goal)
+    intervals2 = _damon.DamonIntervals(*args.monitoring_intervals,
+                                       intervals_goal)
     if not intervals1 == default_intervals:
         return intervals1
     if not intervals2 == default_intervals:
@@ -123,8 +127,17 @@ def damos_filter_with_optional_args(ftype, fmatching, allow, optional_words):
         except Exception as e:
             return None, 'target filter creation failed (%s, %s)' % (
                     optional_words[0], e), 0
-    else:
-        return None, 'unsupported filter type', 0
+    elif ftype == 'hugepage_size':
+        if len(optional_words) < 2:
+            return None, 'no range for hugepage sizes is given', 0
+        try:
+            hugepage_size = _damon.DamonRegion(optional_words[0], optional_words[1])
+        except Exception as e:
+            return None, 'wrong range for hugepage sizes is given (%s, %s)' % (optional_words, e), 0
+        return _damon.DamosFilter(ftype, fmatching, allow=allow,
+                                  hugepage_size=hugepage_size), None, 2
+
+    return None, 'unsupported filter type', 0
 
 def damos_options_to_filter_v2(words):
     '''
@@ -132,8 +145,8 @@ def damos_options_to_filter_v2(words):
     '''
     if len(words) < 2:
         return None, 'wrong number of words: %s' % words, 0
-    if not words[0] in ['allow', 'reject']:
-        return None, 'first word should be allow or reject', 0
+    if not words[0] in ['allow', 'reject', 'deny', 'block']:
+        return None, 'unsupported allow-reject idntifier: %s' % words[0], 0
     allow = words[0] == 'allow'
     nr_consumed_words = 1
     word = words[nr_consumed_words]
@@ -145,7 +158,7 @@ def damos_options_to_filter_v2(words):
     ftype = words[nr_consumed_words]
     nr_consumed_words += 1
 
-    if ftype in ['anon', 'young', 'hugepage']:
+    if ftype in ['anon', 'young', 'unmapped']:
         filter = _damon.DamosFilter(ftype, fmatching, allow=allow)
         return filter, None, nr_consumed_words
     else:
@@ -174,7 +187,7 @@ def convert_damos_filter_v1_to_v2(filter_args):
     if len(optional_args) == 0:
         allow_reject = 'reject'
     else:
-        if optional_args[-1] in ['allow', 'pass', 'reject', 'block']:
+        if optional_args[-1] in ['allow', 'pass', 'reject', 'deny', 'block']:
             if optional_args[-1] in ['allow', 'pass']:
                 allow_reject = 'allow'
             else:
@@ -480,10 +493,9 @@ target_type_unknown = None
 def deduced_target_type(target):
     if target in ['vaddr', 'paddr', 'fvaddr']:
         return target_type_explicit
-    try:
-        subprocess.check_output(['which', target.split()[0]])
+    if _damo_subproc.avail_cmd(target.split()[0]):
         return target_type_cmd
-    except:
+    else:
         pass
     try:
         pid = int(target)
@@ -658,6 +670,11 @@ def set_monitoring_attrs_argparser(parser, hide_help=False):
                         metavar=('<sample>', '<aggr>', '<update>'),
                         help='monitoring intervals (us)'
                         if not hide_help else argparse.SUPPRESS)
+    parser.add_argument(
+            '--monitoring_intervals_goal', nargs=4,
+            metavar=('<access_bp>', '<aggrs>', '<min_sample_us>',
+                     '<max_sample_us>'), default=['0%', '0', '0us', '0us'],
+            help='monitoring intervals auto-tuning goal')
     parser.add_argument('--monitoring_nr_regions_range', nargs=2,
                         metavar=('<min>', '<max>'), default=[10, 1000],
                         help='min/max number of monitoring regions'
@@ -675,7 +692,7 @@ def set_monitoring_argparser(parser, hide_help=False):
                         help='monitoring target address regions'
                         if not hide_help else argparse.SUPPRESS)
     parser.add_argument(
-            '--numa_node', metavar='<node id>', type=int,
+            '--numa_node', metavar='<node id>', type=int, nargs='+',
             help='if target is \'paddr\', limit it to the numa node'
             if not hide_help else argparse.SUPPRESS)
     set_monitoring_attrs_argparser(parser, hide_help)
