@@ -60,6 +60,10 @@ def is_kdamond_running(kdamond_idx):
         return False
     return content.strip() == 'on'
 
+'Return error'
+def update_tuned_intervals(kdamond_idxs):
+    return __write_state_file(kdamond_idxs, 'update_tuned_intervals')
+
 'Return error'
 def update_schemes_stats(kdamond_idxs):
     return __write_state_file(kdamond_idxs, 'update_schemes_stats')
@@ -118,6 +122,19 @@ def write_filter_dir(dir_path, filter_):
         if err is not None:
             return err
 
+    if filter_.hugepage_size is not None:
+        err = _damo_fs.write_file(
+                os.path.join(dir_path, 'min'),
+                '%d' % filter_.hugepage_size.start)
+        if err is not None:
+            return err
+
+        err = _damo_fs.write_file(
+                os.path.join(dir_path, 'max'),
+                '%d' % filter_.hugepage_size.end)
+        if err is not None:
+            return err
+
     if filter_.address_range is not None:
         err = _damo_fs.write_file(
                 os.path.join(dir_path, 'addr_start'),
@@ -168,6 +185,27 @@ def write_filters_dir(dir_path, filters):
             return err
     return None
 
+def is_core_filter(damos_filter):
+    return damos_filter.filter_type in ['addr', 'target']
+
+def write_core_ops_filters_dirs(scheme_dir_path, filters):
+    core_filters = [f for f in filters if is_core_filter(f)]
+    err = write_filters_dir(os.path.join(scheme_dir_path, 'core_filters'),
+                            core_filters)
+    if err is not None:
+        return err
+
+    ops_filters = [f for f in filters if not is_core_filter(f)]
+    err = write_filters_dir(os.path.join(scheme_dir_path, 'ops_filters'),
+                            ops_filters)
+    if err is not None:
+        return err
+
+def write_filters_dirs(scheme_dir_path, filters):
+    if os.path.isdir(os.path.join(scheme_dir_path, 'core_filters')):
+        return write_core_ops_filters_dirs(scheme_dir_path, filters)
+    return write_filters_dir(os.path.join(scheme_dir_path, 'filters'), filters)
+
 def write_watermarks_dir(dir_path, wmarks):
     if wmarks is None:
         # TODO: ensure wmarks is not None
@@ -323,7 +361,7 @@ def write_scheme_dir(dir_path, scheme):
         if err is not None:
             return err
 
-    err = write_filters_dir(os.path.join(dir_path, 'filters'), scheme.filters)
+    err = write_filters_dirs(dir_path, scheme.filters)
     if err is not None:
         return err
 
@@ -392,6 +430,24 @@ def write_targets_dir(dir_path, targets):
             return err
     return None
 
+def write_monitoring_intervals_goal_dir(dir_path, goal):
+    err = _damo_fs.write_file(os.path.join(dir_path, 'access_bp'), '%d' %
+                              goal.access_bp)
+    if err is not None:
+        return err
+    err = _damo_fs.write_file(os.path.join(dir_path, 'aggrs'), '%d' %
+                              goal.aggrs)
+    if err is not None:
+        return err
+    err = _damo_fs.write_file(os.path.join(dir_path, 'min_sample_us'), '%d' %
+                              goal.min_sample_us)
+    if err is not None:
+        return err
+    err = _damo_fs.write_file(os.path.join(dir_path, 'max_sample_us'), '%d' %
+                              goal.max_sample_us)
+    if err is not None:
+        return err
+
 def write_monitoring_attrs_dir(dir_path, context):
     err = _damo_fs.write_file(
             os.path.join(dir_path, 'intervals', 'sample_us'),
@@ -411,6 +467,13 @@ def write_monitoring_attrs_dir(dir_path, context):
     if err is not None:
         return err
 
+    intervals_goal_path = os.path.join(dir_path, 'intervals', 'intervals_goal')
+    if os.path.isdir(intervals_goal_path):
+        err = write_monitoring_intervals_goal_dir(
+                intervals_goal_path, context.intervals.intervals_goal)
+        if err is not None:
+            return err
+
     err = _damo_fs.write_file(
             os.path.join(dir_path, 'nr_regions', 'min'),
             '%d' % context.nr_regions.minimum)
@@ -557,6 +620,11 @@ def files_content_to_damos_filter(files_content):
     elif 'pass' in files_content:
         allow = files_content['pass'].strip()
 
+    hugepage_size=None
+    if 'min' in files_content and 'max' in files_content:
+        hugepage_size=_damon.DamonRegion(files_content['min'].strip(),
+                                         files_content['max'].strip())
+
     return _damon.DamosFilter(
         files_content['type'].strip(),
         files_content['matching'].strip(),
@@ -567,12 +635,22 @@ def files_content_to_damos_filter(files_content):
         if 'addr_start' in files_content and 'addr_end' in files_content
         else None,
         files_content['damon_target_idx']
-        if 'damon_target_idx' in files_content else None)
+        if 'damon_target_idx' in files_content else None,
+        hugepage_size=hugepage_size)
 
-def files_content_to_damos_filters(files_content):
-    return [files_content_to_damos_filter(filter_kv)
-            for filter_kv in numbered_dirs_content(
-                files_content, 'nr_filters')]
+def files_content_to_damos_filters(scheme_files_content):
+    filters = []
+    if 'core_filters' in scheme_files_content:
+        filters += [files_content_to_damos_filter(filter_kv)
+                    for filter_kv in numbered_dirs_content(
+                        scheme_files_content['core_filters'], 'nr_filters')]
+        filters += [files_content_to_damos_filter(filter_kv)
+                    for filter_kv in numbered_dirs_content(
+                        scheme_files_content['ops_filters'], 'nr_filters')]
+    filters += [files_content_to_damos_filter(filter_kv)
+                for filter_kv in numbered_dirs_content(
+                    scheme_files_content['filters'], 'nr_filters')]
+    return filters
 
 def files_content_to_damos_stats(files_content):
     return _damon.DamosStats(
@@ -603,7 +681,7 @@ def files_content_to_scheme(files_content):
                 if 'apply_interval_us' in files_content else None,
             files_content_to_quotas(files_content['quotas']),
             files_content_to_watermarks(files_content['watermarks']),
-            files_content_to_damos_filters(files_content['filters'])
+            files_content_to_damos_filters(files_content)
                 if 'filters' in files_content else [],
             files_content_to_damos_stats(files_content['stats']),
             files_content_to_damos_tried_regions(
@@ -629,10 +707,18 @@ def files_content_to_target(files_content):
 def files_content_to_context(files_content):
     mon_attrs_content = files_content['monitoring_attrs']
     intervals_content = mon_attrs_content['intervals']
+    if 'intervals_goal' in intervals_content:
+        kvpairs = intervals_content['intervals_goal']
+        intervals_goal = _damon.DamonIntervalsGoal(
+                int(kvpairs['access_bp']), int(kvpairs['aggrs']),
+                int(kvpairs['min_sample_us']), int(kvpairs['max_sample_us']))
+    else:
+        intervals_goal = _damon.DamonIntervalsGoal()
     intervals = _damon.DamonIntervals(
             int(intervals_content['sample_us']),
             int(intervals_content['aggr_us']),
-            int(intervals_content['update_us']))
+            int(intervals_content['update_us']),
+            intervals_goal)
     nr_regions_content = mon_attrs_content['nr_regions']
     nr_regions = _damon.DamonNrRegionsRange(
             int(nr_regions_content['min']),
@@ -767,6 +853,10 @@ def infer_damon_version():
                                 )])])]
     err = stage_kdamonds(kdamonds)
     if err is None:
+        if os.path.isdir(
+                os.path.join(ctx_dir_of(0, 0),
+                             'monitoring_attrs', 'intervals', 'intervals_goal')):
+            version = '>v6.14'
         if os.path.isfile(os.path.join(scheme_dir_of(0, 0, 0), 'stats',
                                        'sz_ops_filter_passed')):
             version = '>v6.13'
@@ -876,13 +966,25 @@ def update_supported_features():
                          'stats', 'sz_ops_filter_passed')):
         feature_supports['sz_ops_filter_passed'] = True
 
-    if os.path.isfile(
-            os.path.join(scheme_dir_of(0, 0, 0), 'filters', '0', 'pass')):
-        feature_supports['pass_filter'] = True
     if os.path.isfile(
             os.path.join(scheme_dir_of(0, 0, 0), 'filters', '0', 'allow')):
         feature_supports['allow_filter'] = True
 
+    if os.path.isfile(
+            os.path.join(scheme_dir_of(0, 0, 0), 'filters', '0', 'min')):
+        feature_supports['schemes_filters_hugepage_size'] = True
+
+    if os.path.isdir(
+            os.path.join(ctx_dir_of(0, 0),
+                         'monitoring_attrs', 'intervals', 'intervals_goal')):
+        feature_supports['intervals_goal'] = True
+
+    if os.path.isdir(
+            os.path.join(scheme_dir_of(0, 0, 0), 'core_filters')):
+        feature_supports['schemes_filters_core_ops_dirs'] = True
+
+    # todo: check unmapped filter support by trying writing it to filter type
+
     avail_ops, err = _avail_ops()
     if err == None:
         for ops in ['vaddr', 'paddr', 'fvaddr']:
