@@ -6,6 +6,7 @@ import json
 import _damo_yaml
 import _damon
 import _damon_args
+import damo_report_damon
 
 def main(args):
     _damon.ensure_root_permission()
@@ -47,7 +48,17 @@ def main(args):
         old_kdamonds[kdamond_idx] = kdamonds[0]
         kdamonds = old_kdamonds
 
-    kvpairs = {'kdamonds': [k.to_kvpairs(args.raw) for k in kdamonds]}
+    kvpairs = {'kdamonds':
+               [k.to_kvpairs(args.raw, args.omit_defaults, params_only=True)
+                for k in kdamonds]}
+    if args.format == 'report':
+        if args.out is not None:
+            print('--out and report format cannot be used together')
+            exit(1)
+        damo_report_damon.pr_kdamonds(
+                kdamonds, json_format=False, raw_nr=args.raw, show_cpu=False)
+        return
+
     if args.format == 'json':
         text = json.dumps(kvpairs, indent=4)
     elif args.format == 'yaml':
@@ -66,7 +77,7 @@ def set_argparser(parser):
     parser.description = ' '.join([
         'format DAMON parameters'])
     parser.add_argument(
-            '--format', choices=['json', 'yaml'], default='json',
+            '--format', choices=['json', 'yaml', 'report'], default='json',
             help='format of the output')
     parser.add_argument(
             '--raw', action='store_true',
@@ -82,3 +93,5 @@ def set_argparser(parser):
             help='replace a kdamond of given index in the given file')
     parser.add_argument(
             '--out', metavar='<file>', help='save the output to the given file')
+    parser.add_argument( '--omit_defaults', action='store_true',
+                        help='omit default parameters')
