@@ -10,6 +10,7 @@ import signal
 import subprocess
 import time
 
+import _damo_fmt_str
 import _damo_records
 import _damon
 import _damon_args
@@ -59,7 +60,8 @@ def handle_args(args):
     if os.path.isfile(args.out):
         os.rename(args.out, args.out + '.old')
 
-    if args.footprint is True:
+
+    if 'mem_footprint' in args.do_record:
         footprint_file_path = '%s.mem_footprint' % args.out
         if os.path.isfile(footprint_file_path):
             os.rename(footprint_file_path, footprint_file_path + '.old')
@@ -82,11 +84,12 @@ def mk_handle(args, kdamonds, monitoring_intervals):
             file_permission=args.output_permission,
             monitoring_intervals=monitoring_intervals,
             # for perf profile
-            do_profile=args.profile is True,
+            do_profile='cpu_profile' in args.do_record,
             # for children processes recording and memory footprint
             kdamonds=kdamonds, add_child_tasks=args.include_child_tasks,
-            record_mem_footprint=args.footprint,
-            record_vmas=args.vmas, timeout=args.timeout)
+            record_mem_footprint='mem_footprint' in args.do_record,
+            record_vmas='vmas' in args.do_record, record_proc_stats=True,
+            timeout=args.timeout)
 
     if args.snapshot is not None:
         handle.tracepoint = None
@@ -97,12 +100,32 @@ def mk_handle(args, kdamonds, monitoring_intervals):
                 for cidx, ctx in enumerate(kd.contexts):
                     for sidx, scheme in enumerate(ctx.schemes):
                         tried_regions_of.append([kidx, cidx, sidx])
+
+        dfilters, err = _damon_args.damos_options_to_filters(
+                args.snapshot_damos_filter)
+        if err is not None:
+            print('wrong --snapshot_damos_filters (%s)' % err)
+            cleanup_exit(1)
+
+        record_filter, err = _damo_records.args_to_filter(args)
+        if err is not None:
+            print('record filter creation fail (%s)' % err)
+            cleanup_exit(1)
+
         handle.snapshot_request = _damo_records.RecordGetRequest(
                 tried_regions_of=tried_regions_of, record_file=None,
-                record_filter=None, total_sz_only=False,
+                snapshot_damos_filters=dfilters,
+                record_filter=record_filter, total_sz_only=False,
                 dont_merge_regions=False)
-        handle.snapshot_interval_sec = args.snapshot[0]
-        handle.snapshot_count = args.snapshot[1]
+        handle.snapshot_interval_sec = _damo_fmt_str.text_to_sec(
+                args.snapshot[0])
+        handle.snapshot_count = _damo_fmt_str.text_to_nr(args.snapshot[1])
+
+    if not 'access' in args.do_record:
+        handle.tracepoint = None
+        handle.snapshot_request = None
+    if not 'proc_stats' in args.do_record:
+        handle.proc_stats = None
 
     return handle
 
@@ -165,15 +188,16 @@ def set_argparser(parser):
                         help='record accesses of child processes')
     parser.add_argument('--schemes_target_regions', action='store_true',
                         help='record schemes tried to be applied regions')
-    parser.add_argument('--no_profile', action='store_false', dest='profile',
-                        help='do not record profiling information')
-    parser.add_argument('--no_footprint', action='store_false',
-                        dest='footprint',
-                        help='do not record memory footprint')
-    parser.add_argument('--no_vmas', action='store_false', dest='vmas',
-                        help='record virtual memory areas (/proc/<pid>/maps)')
     parser.add_argument('--snapshot', metavar=('<delay>', '<count>'), nargs=2,
-                        type=float, help='record accesses as snapshots')
+                        help='record accesses as snapshots')
     parser.add_argument('--timeout', type=float, metavar='<seconds>',
                         help='stop recording after the given seconds')
+    parser.add_argument('--do_record', nargs='+',
+                        default=['access', 'cpu_profile', 'mem_footprint',
+                                 'vmas', 'proc_stats'],
+                        choices=['access', 'cpu_profile', 'mem_footprint',
+                                 'vmas', 'proc_stats'],
+                        help='what to do record')
+    _damo_records.set_snapshot_damos_filters_option(parser)
+    _damo_records.set_filter_argparser(parser)
     return parser
