@@ -5,6 +5,8 @@ import copy
 import json
 import math
 import os
+import signal
+import time
 
 import _damo_ascii_color
 import _damo_fmt_str
@@ -28,27 +30,31 @@ class Formatter:
 
 record_formatters = [
         Formatter('<kdamond index>',
-            lambda record, raw: '%s' % record.kdamond_idx,
+            lambda record, fmt: '%s' % record.kdamond_idx,
             'index of the record\'s kdamond'),
         Formatter('<context index>',
-            lambda record, raw: '%s' % record.context_idx,
+            lambda record, fmt: '%s' % record.context_idx,
             'index of the record\'s DAMON context'),
         Formatter('<scheme index>',
-            lambda record, raw: '%s' % record.scheme_idx,
+            lambda record, fmt: '%s' % record.scheme_idx,
             'index of the record\'s DAMOS scheme'),
         Formatter('<target id>',
-            lambda record, raw: '%s' % record.target_id,
+            lambda record, fmt: '%s' % record.target_id,
             'index of the record\'s DAMON target'),
         Formatter('<abs start time>',
-            lambda record, raw:
-            _damo_fmt_str.format_time_ns(record.snapshots[0].start_time, raw),
+            lambda record, fmt:
+            _damo_fmt_str.format_time_ns(record.snapshots[0].start_time,
+                                         fmt.raw_number),
             'absolute time of the start of the record'),
         Formatter('<duration>',
-            lambda record, raw:
+            lambda record, fmt:
             _damo_fmt_str.format_time_ns(
                 record.snapshots[-1].end_time - record.snapshots[0].start_time,
-                raw),
+                fmt.raw_number),
             'duration of the record'),
+        Formatter('<format strings>',
+                  lambda record, fmt: format_strings(fmt),
+                  'current format strings')
         ]
 
 snapshot_formatters = [
@@ -116,6 +122,10 @@ snapshot_formatters = [
                   lambda snapshot, record, fmt:
                   heatmap_str(snapshot, record, fmt),
                   'heatmap of the snapshot'),
+        Formatter('<filters passed heatmap>',
+                  lambda snapshot, record, fmt:
+                  df_passed_heatmap_str(snapshot, record, fmt),
+                  'heatmap of the snapshot for filter-passed regions'),
         Formatter(
             '<filters passed type>',
             lambda snapshot, record, fmt: filters_pass_type_of(record),
@@ -130,6 +140,22 @@ snapshot_formatters = [
                 lambda snapshot, record, fmt:
                 positive_access_sample_ratio(snapshot, record, fmt),
                 'positive access samples ratio'),
+        Formatter(
+                '<estimated memory bandwidth>',
+                lambda snapshot, record, fmt:
+                estimated_mem_bw(snapshot, record, fmt),
+                'estimated memory bandwidth'),
+        Formatter(
+                '<filters passed estimated memory bandwidth>',
+                lambda snapshot, record, fmt:
+                estimated_mem_bw(snapshot, record, fmt,
+                                 filter_passed_only=True),
+                'estimated memory bandwidth'),
+        Formatter(
+                '<intervals tuning status>',
+                lambda snapshot, record, fmt:
+                intervals_tuning_status(snapshot, record, fmt),
+                'intervals tuning status'),
         ]
 
 region_formatters = [
@@ -186,7 +212,17 @@ default_record_head_format = 'kdamond <kdamond index> / context <context index>
 default_snapshot_head_format = 'monitored time: [<start time>, <end time>] (<duration>)\n<heatmap>'
 default_snapshot_head_format_without_heatmap = 'monitored time: [<start time>, <end time>] (<duration>)'
 default_region_format = '<index> addr <start address> size <size> access <access rate> age <age>'
-default_snapshot_tail_format = 'total size: <total bytes>'
+default_snapshot_tail_format = 'memory bw estimate: <estimated memory bandwidth>\ntotal size: <total bytes>'
+default_snapshot_tail_format_filter_installed = 'memory bw estimate: <estimated memory bandwidth>  df-passed: <filters passed estimated memory bandwidth>\ntotal size: <total bytes>  df-passed <filters passed bytes>'
+
+def format_strings(fmt):
+    return '\n'.join([
+        'record head: "%s"' % fmt.format_record_head,
+        'snapshot head: "%s"' % fmt.format_snapshot_head,
+        'region: "%s"' % fmt.format_region,
+        'snapshot tail: "%s"' % fmt.format_snapshot_tail,
+        'record tail: "%s"' % fmt.format_record_tail,
+        ])
 
 def filters_passed_bytes(snapshot, fmt):
     bytes = 0
@@ -206,11 +242,45 @@ def positive_access_sample_ratio(snapshot, record, fmt):
     return _damo_fmt_str.format_percent(
             nr_samples * 100 / max_samples, fmt.raw_number)
 
+def intervals_tuning_status(snapshot, record, fmt):
+    max_samples_per_region = record.intervals.aggr / record.intervals.sample
+    max_observation = 0
+    observation = 0
+    for region in snapshot.regions:
+        region.nr_accesses.add_unset_unit(record.intervals)
+        max_observation += max_samples_per_region * region.size()
+        observation += region.size() * region.nr_accesses.samples
+    return '%s (%s / %s) %s %s' % (
+            _damo_fmt_str.format_percent(observation * 100 / max_observation,
+                                         fmt.raw_number),
+            _damo_fmt_str.format_sz(observation, fmt.raw_number),
+            _damo_fmt_str.format_sz(max_observation, fmt.raw_number),
+            _damo_fmt_str.format_time_us(record.intervals.sample,
+                                         fmt.raw_number),
+            _damo_fmt_str.format_time_us(record.intervals.aggr,
+                                         fmt.raw_number),
+            )
+
+def estimated_mem_bw(snapshot, record, fmt, filter_passed_only=False):
+    access_bytes = 0
+    for region in snapshot.regions:
+        if filter_passed_only is True:
+            bytes = region.sz_filter_passed
+        else:
+            bytes = region.size()
+        access_bytes += bytes * region.nr_accesses.samples
+    bw_per_sec = access_bytes / (record.intervals.aggr / 1000000)
+    return '%s per second' % _damo_fmt_str.format_sz(
+            bw_per_sec, fmt.raw_number)
+
 def filters_pass_type_of(record):
     ops_filters = [f for f in record.scheme_filters if f.handled_by_ops()]
     if len(ops_filters) == 0:
         return 'n/a'
-    return ', '.join(['%s' % f for f in ops_filters])
+    text = ', '.join(['%s' % f for f in ops_filters])
+    if ops_filters[-1].allow is True:
+        text += '\n# WARN: Allow filters at the end of the list means nothing'
+    return text
 
 def temperature_sz_hist_str(snapshot, record, fmt, df_passed_sz):
     raw = fmt.raw_number
@@ -396,9 +466,13 @@ def heatmap_str(snapshot, record, fmt):
     if len(snapshot.regions) == 0:
         return 'n/a (no region)'
     raw = fmt.raw_number
-    total_sz = 0
-    for region in snapshot.regions:
-        total_sz += region.size()
+    static = fmt.snapshot_heatmap_static
+    if static:
+        total_sz = snapshot.regions[-1].end - snapshot.regions[0].start
+    else:
+        total_sz = 0
+        for region in snapshot.regions:
+            total_sz += region.size()
     map_length = fmt.snapshot_heatmap_width
     sz_unit = total_sz / map_length
 
@@ -414,7 +488,8 @@ def heatmap_str(snapshot, record, fmt):
             next_region = region_after(end, snapshot.regions)
             if next_region is None:
                 break
-            start = next_region.start
+            start = next_region.start // sz_unit * sz_unit
+            pixels[-1].end = start
         else:
             pixel = pixels[-1]
             if min_temperature is None or pixel.temperature < min_temperature:
@@ -428,10 +503,16 @@ def heatmap_str(snapshot, record, fmt):
     # single region?
     if temperature_unit == 0:
         temperature_unit = max_temperature / max_color_level
+    if temperature_unit == 0:
+        temperature_unit = 1
     dots = []
     for pixel in pixels:
         if pixel.is_void is True:
-            dots.append('[...]')
+            nr_dots = (pixel.end - pixel.start) / sz_unit
+            if not static and nr_dots > 4:
+                dots.append('[...]')
+            else:
+                dots += ['.'] * int(nr_dots)
             continue
         temp_level = int(
                 (pixel.temperature - min_temperature) / temperature_unit)
@@ -444,6 +525,19 @@ def heatmap_str(snapshot, record, fmt):
             _damo_fmt_str.format_sz(sz_unit, raw))
     return '%s\n%s' % (dots, comment)
 
+def df_passed_heatmap_str(snapshot, record, fmt):
+    regions = []
+    for region in snapshot.regions:
+        cp_region = copy.deepcopy(region)
+        if region.sz_filter_passed == 0:
+            continue
+        regions.append(cp_region)
+    return heatmap_str(
+            _damo_records.DamonSnapshot(
+                snapshot.start_time, snapshot.end_time, regions,
+                total_bytes=None),
+            record, fmt)
+
 def rescale(val, orig_scale_minmax, new_scale_minmax, logscale=True):
     '''Return a value in new scale
 
@@ -693,7 +787,7 @@ def format_output(template, formatters, fmt, record, snapshot=None,
         if template.find(formatter.keyword) == -1:
             continue
         if formatters == record_formatters:
-            txt = formatter.format_fn(record, fmt.raw_number)
+            txt = formatter.format_fn(record, fmt)
         elif formatters == snapshot_formatters:
             txt = formatter.format_fn(snapshot, record, fmt)
         elif formatters == region_formatters:
@@ -819,7 +913,7 @@ def pr_records_raw_form(records, raw_number):
     lines.append('')
     _damo_print.pr_with_pager_if_needed('\n'.join(lines))
 
-def pr_records(fmt, records):
+def pr_records(fmt, records, dont_use_pager):
     if fmt.json:
         _damo_print.pr_with_pager_if_needed(
                 json.dumps([r.to_kvpairs(fmt.raw_number) for r in records],
@@ -827,7 +921,11 @@ def pr_records(fmt, records):
     elif fmt.raw:
         pr_records_raw_form(records, fmt.raw_number)
     else:
-        _damo_print.pr_with_pager_if_needed(fmt_records(fmt, records))
+        to_show = fmt_records(fmt, records)
+        if dont_use_pager:
+            print(to_show)
+        else:
+            _damo_print.pr_with_pager_if_needed(fmt_records(fmt, records))
 
 class ReportFormat:
     sort_regions_by = None
@@ -843,6 +941,7 @@ class ReportFormat:
 
     snapshot_heatmap_width = None
     snapshot_heatmap_colorset = None
+    snapshot_heatmap_static = None
 
     region_box_values = None
     region_box_min_max_height = None
@@ -873,6 +972,7 @@ class ReportFormat:
         self.format_snapshot_tail = args.format_snapshot_tail
         self.format_region = args.format_region
         self.snapshot_heatmap_width = args.snapshot_heatmap_width
+        self.snapshot_heatmap_static = args.snapshot_heatmap_static
         self.snapshot_heatmap_colorset = args.snapshot_heatmap_colorset
         self.region_box_values = args.region_box_values
         self.region_box_min_max_height = args.region_box_min_max_height
@@ -904,6 +1004,7 @@ class ReportFormat:
                 'format_snapshot_tail': self.format_snapshot_tail,
                 'format_region': self.format_region,
                 'snapshot_heatmap_width': self.snapshot_heatmap_width,
+                'snapshot_heatmap_static': self.snapshot_heatmap_static,
                 'snapshot_heatmap_colorset': self.snapshot_heatmap_colorset,
                 'region_box_values': self.region_box_values,
                 'region_box_min_max_height': self.region_box_min_max_height,
@@ -938,6 +1039,11 @@ class ReportFormat:
             self.snapshot_heatmap_width = kvpairs['snapshot_heatmap_width']
         else:
             self.snapshot_heatmap_width = 'gray'
+        # snapshot_heatmap_static introduced after v2.6.7
+        if 'snapshot_heatmap_static' in kvpairs:
+            self.snapshot_heatmap_static = kvpairs['snapshot_heatmap_static']
+        else:
+            self.snapshot_heatmap_static = False
         self.region_box_values = kvpairs['region_box_values']
         self.region_box_min_max_height = kvpairs['region_box_min_max_height']
         self.region_box_min_max_length = kvpairs['region_box_min_max_length']
@@ -987,6 +1093,21 @@ def set_formats(args, records):
         fmt.region_box_colorset = 'emotion'
     elif args.style in ['temperature-sz-hist', 'recency-sz-hist']:
         set_formats_hist_style(args, fmt, records)
+    elif args.style == 'cold':
+        fmt.format_region = '<box> <size> access <access rate> <age>'
+        fmt.region_box_min_max_height = [1, 1]
+        fmt.region_box_min_max_length = [1, 40]
+        fmt.region_box_align = 'right'
+        fmt.region_box_colorset = 'emotion'
+        fmt.sort_regions_by = ['temperature']
+    elif args.style == 'hot':
+        fmt.format_region = '<box> <size> access <access rate> <age>'
+        fmt.region_box_min_max_height = [1, 1]
+        fmt.region_box_min_max_length = [1, 40]
+        fmt.region_box_align = 'right'
+        fmt.region_box_colorset = 'emotion'
+        fmt.sort_regions_by = ['temperature']
+        fmt.sort_regions_dsc = ['temperature']
 
     fmt.region_box_values = [v if v != 'none' else None
             for v in args.region_box_values]
@@ -1022,7 +1143,7 @@ def set_formats(args, records):
     if fmt.format_snapshot_tail == default_snapshot_tail_format:
         for record in records:
             if len(record.scheme_filters) > 0:
-                fmt.format_snapshot_tail += '  df-passed: <filters passed bytes>'
+                fmt.format_snapshot_tail = default_snapshot_tail_format_filter_installed
                 break
 
     if fmt.format_snapshot_head == None:
@@ -1037,6 +1158,7 @@ def set_formats(args, records):
             fmt.format_snapshot_head = 'heatmap: <heatmap>'
     if '<filters passed bytes>' in fmt.format_region:
         fmt.format_snapshot_head += '\n# damos filters (df): <filters passed type>'
+        fmt.format_snapshot_head += '\ndf-pass: <filters passed heatmap>'
 
     return fmt
 
@@ -1100,11 +1222,14 @@ def translate_records_to_cache_space(records, cs, cw, cl):
             snapshot.regions = translate_regions_to_cache_space(
                     snapshot.regions, cache_spec)
 
-def main(args):
-    handled = handle_ls_keywords(args)
-    if handled:
-        return
+signal_received = False
 
+def sighandler(signum, frame):
+    global signal_received
+    print('\nsignal %s received' % signum)
+    signal_received = True
+
+def read_and_show(args):
     record_filter, err = _damo_records.args_to_filter(args)
     if err != None:
         print(err)
@@ -1122,49 +1247,83 @@ def main(args):
         print('wrong --snapshot_damos_filter (%s)' % err)
         exit(1)
 
-    records, err = _damo_records.get_records(
-                tried_regions_of=args.tried_regions_of,
-                record_file=args.input_file, snapshot_damos_filters=dfilters,
-                record_filter=record_filter,
-                total_sz_only=args.total_sz_only,
-                dont_merge_regions=args.dont_merge_regions)
-    if err != None:
-        print(err)
+    if args.repeat is None:
+        repeat_delay = 0
+        repeat_count = 1
+    elif len(args.repeat) == 0:
+        repeat_delay = 1
+        repeat_count = -1
+    elif len(args.repeat) == 2:
+        repeat_delay = _damo_fmt_str.text_to_sec(args.repeat[0])
+        repeat_count = _damo_fmt_str.text_to_nr(args.repeat[1])
+    else:
+        print('--repeat receives only zero or two arguments')
         exit(1)
 
-    if len([r for r in records if r.intervals is None]) != 0:
-        if not args.json and not args.raw_form:
-            print('some records lack the intervals information')
+    signal.signal(signal.SIGINT, sighandler)
+    global signal_received
+
+    read_show_count = 0
+    while read_show_count < repeat_count or repeat_count == -1:
+        if signal_received is True:
+            break
+        records, err = _damo_records.get_records(
+                    tried_regions_of=args.tried_regions_of,
+                    record_file=args.input_file, snapshot_damos_filters=dfilters,
+                    record_filter=record_filter,
+                    total_sz_only=args.total_sz_only,
+                    dont_merge_regions=args.dont_merge_regions)
+        if err != None:
+            print(err)
             exit(1)
+        if signal_received is True:
+            break
 
-    if args.format is not None:
-        fmt_string = args.format
-        if os.path.isfile(fmt_string):
-            with open(fmt_string, 'r') as f:
-                fmt_string = f.read()
-        fmt = ReportFormat.from_kvpairs(json.loads(fmt_string))
-    else:
-        fmt = set_formats(args, records)
+        if len([r for r in records if r.intervals is None]) != 0:
+            if not args.json and not args.raw_form:
+                print('some records lack the intervals information')
+                exit(1)
 
-    if args.on_cache is not None:
-        sz_cache = _damo_fmt_str.text_to_bytes(args.on_cache[0])
-        ways_cache = _damo_fmt_str.text_to_nr(args.on_cache[1])
-        sz_cache_line = _damo_fmt_str.text_to_bytes(args.on_cache[2])
-        translate_records_to_cache_space(
-                records, sz_cache, ways_cache, sz_cache_line)
+        if args.format is not None:
+            fmt_string = args.format
+            if os.path.isfile(fmt_string):
+                with open(fmt_string, 'r') as f:
+                    fmt_string = f.read()
+            fmt = ReportFormat.from_kvpairs(json.loads(fmt_string))
+        else:
+            fmt = set_formats(args, records)
 
-    for record in records:
-        try:
-            pr_records(fmt, records)
-        except BrokenPipeError as e:
-            # maybe user piped to 'less' like pager, and quit from it
-            pass
+        if args.on_cache is not None:
+            sz_cache = _damo_fmt_str.text_to_bytes(args.on_cache[0])
+            ways_cache = _damo_fmt_str.text_to_nr(args.on_cache[1])
+            sz_cache_line = _damo_fmt_str.text_to_bytes(args.on_cache[2])
+            translate_records_to_cache_space(
+                    records, sz_cache, ways_cache, sz_cache_line)
+
+        for record in records:
+            try:
+                pr_records(fmt, records,
+                           dont_use_pager = args.repeat is not None)
+            except BrokenPipeError as e:
+                # maybe user piped to 'less' like pager, and quit from it
+                pass
+
+        time.sleep(repeat_delay)
+        read_show_count += 1
+
+def main(args):
+    handled = handle_ls_keywords(args)
+    if handled:
+        return
+
+    read_and_show(args)
 
 def add_fmt_args(parser, hide_help=False):
     # how to show, in simple selection
     parser.add_argument(
             '--style', choices=['detailed', 'simple-boxes',
-                                'temperature-sz-hist', 'recency-sz-hist'],
+                                'temperature-sz-hist', 'recency-sz-hist',
+                                'cold', 'hot'],
             default='detailed',
             help='output format selection among pre-configures ones')
     # how to show, in highly tunable way
@@ -1213,6 +1372,9 @@ def add_fmt_args(parser, hide_help=False):
             '--snapshot_heatmap_colorset', default='gray',
             choices=_damo_ascii_color.colorsets.keys(),
             help='snapshot heatmap colorset')
+    parser.add_argument(
+            '--snapshot_heatmap_static', action='store_true',
+            help='draw snapshot heatmap as static')
     parser.add_argument('--format_region', metavar='<template>',
                         default=default_region_format,
                         help='output format to show for each memory region'
@@ -1314,3 +1476,6 @@ def set_argparser(parser):
             '--on_cache', nargs=3,
             metavar=('<cache size>', '<cache ways>', '<cache line size>'),
             help='visualize access patterns on a virtual cache (EXPERIMENTAL)')
+    parser.add_argument(
+            '--repeat', nargs='*', metavar=('<delay>', '<count>'),
+            help='repeat <count> times with <delay> time interval')
