@@ -19,8 +19,8 @@ def main(args):
     don't use the approach to keep this file as a place to add deprecation
     message later.
     '''
-    _damo_deprecation_notice.will_be_deprecated(
-            feature='"damo status"', deadline='2025-01-20',
+    _damo_deprecation_notice.deprecated(
+            feature='"damo status"', deadline='2025-02-26',
             additional_notice='Use "damo report damon" instead.')
     return damo_report_damon.main(args)
 
